@@ -1,7 +1,7 @@
 /*
  * abcm2ps: a program to typeset tunes written in abc format using PostScript
  *
- * Copyright (C) 1998-2007 Jean-Franois Moine
+ * Copyright (C) 1998-2008 Jean-Franois Moine
  *
  * Adapted from abc2ps-1.2.5:
  *  Copyright (C) 1996,1997  Michael Methfessel
@@ -20,7 +20,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Original page:
+ * Original site:
  *	http://moinejf.free.fr/
  *
  * Original abc2ps:
@@ -72,8 +72,9 @@ time_t mtime;			/* last modification time of the input file */
 int s_argc;			/* command line arguments */
 char **s_argv;
 
-struct WHISTLE_S whistle_tb[MAXWHISTLE];
-int nwhistle;
+struct tblt_s *tblts[MAXTBLT];
+struct cmdtblt_s cmdtblts[MAXCMDTBLT];
+int ncmdtblt;
 
 /* -- local variables -- */
 
@@ -81,8 +82,8 @@ static int def_fmt_done = 0;	/* default format read */
 static struct SYMBOL notitle;
 
 /* memory arena (for clrarena, lvlarena & getarena) */
-#define MAXAREAL 4		/* max area levels:
-				 * 0; global, 1: tune, 2: output, 3: output line */
+#define MAXAREAL 3		/* max area levels:
+				 * 0; global, 1: file, 2: tune generation */
 static int str_level;		/* current arena level */
 static struct str_a {
 	char	str[4096];	/* memory area */
@@ -105,7 +106,7 @@ static void usage(void);
 #ifdef linux
 static void wherefmtdir(void);
 #endif
-static void whistle_parse(char *p);
+static struct cmdtblt_s *cmdtblt_parse(char *p);
 static void write_version(void);
 
 /* -- main program -- */
@@ -113,8 +114,7 @@ int main(int argc,
 	 char **argv)
 {
 	int j;
-	char *p, *sel;
-	char c, *aaa;
+	char *p, *sel, c, *aaa;
 
 	fprintf(stderr, "abcm2ps-" VERSION " (" VDATE ")\n");
 	if (argc <= 1)
@@ -125,7 +125,6 @@ int main(int argc,
 	clrarena(0);			/* global description */
 	clrarena(1);			/* tune description */
 	clrarena(2);			/* tune generation */
-	clrarena(3);			/* line generation */
 	clear_buffer();
 	abc_init((void *(*)(int size)) getarena, /* alloc */
 		0,				/* free */
@@ -151,11 +150,15 @@ int main(int argc,
 	while (--argc > 0) {
 		argv++;
 		p = *argv;
-		if (*p == 0)
+		if ((c = *p) == '\0')
 			continue;
-		if (*p == '+') {	/* switch off flags with '+' */
+		if (c == '-' && p[1] != '-' && p[strlen(p) - 1] == '-')
+			c = '+';	/* switch off flags with '-x-' */
+		if (c == '+') {		/* switch off flags with '+' */
 			while (*++p != '\0') {
 				switch (*p) {
+				case '-':
+					break;
 				case 'B':
 					cfmt.barsperstaff = 0;
 					lock_fmt(&cfmt.barsperstaff);
@@ -186,7 +189,9 @@ int main(int argc,
 					cfmt.musiconly = 0;
 					lock_fmt(&cfmt.musiconly);
 					break;
-				case 'N': pagenumbers = 0; break;
+				case 'N':
+					pagenumbers = 0;
+					break;
 				case 'n':
 					cfmt.writehistory = 0;
 					lock_fmt(&cfmt.writehistory);
@@ -198,11 +203,30 @@ int main(int argc,
 					cfmt.printtempo = 0;
 					lock_fmt(&cfmt.printtempo);
 					break;
+				case 'T': {
+					struct cmdtblt_s *cmdtblt;
+					aaa = p + 1;
+					if (*aaa == '\0') {
+						if (argc > 1
+						    && argv[1][0] != '-') {
+							aaa = *++argv;
+							argc--;
+						}
+					} else {
+						while (p[1] != '\0')	/* stop */
+							p++;
+						if (*p == '-')
+							*p-- = '\0';	/* (not clean) */
+					}
+					cmdtblt = cmdtblt_parse(aaa);
+					if (cmdtblt != 0)
+						cmdtblt->active = 0;
+					break;
+				   }
 				case 'x':
 					cfmt.withxrefs = 0;
 					lock_fmt(&cfmt.withxrefs);
 					break;
-				case 'W': nwhistle = 0; break;
 				case '0':
 					cfmt.splittune = 0;
 					lock_fmt(&cfmt.splittune);
@@ -222,8 +246,8 @@ int main(int argc,
 			continue;
 		}
 
-		if (*p == '-') {	     /* interpret a flag with '-'*/
-			if (p[1] == '\0') {
+		if (c == '-') {		     /* interpret a flag with '-' */
+			if (p[1] == '\0') {		/* '-' alone */
 				if (in_fname != 0) {
 					output_file(sel);
 					sel = 0;
@@ -231,7 +255,7 @@ int main(int argc,
 				in_fname = "";		/* read from stdin */
 				continue;
 			}
-			if (p[1] == '-') {
+			if (p[1] == '-') {		/* long argument */
 				p += 2;
 				if (--argc <= 0) {
 					fprintf(stderr,
@@ -242,8 +266,8 @@ int main(int argc,
 				interpret_fmt_line(p, *argv, 1);
 				continue;
 			}
-			while (*++p != '\0') {
-				switch (c = *p) {
+			while ((c = *++p) != '\0') {
+				switch (*p) {
 
 					/* simple flags */
 				case 'c':
@@ -347,8 +371,8 @@ int main(int argc,
 				case 'm':
 				case 'O':
 				case 's':
+				case 'T':
 				case 'w':
-				case 'W':
 					aaa = p + 1;
 					if (*aaa == '\0') {
 						aaa = *++argv;
@@ -381,34 +405,24 @@ int main(int argc,
 					}
 
 					switch (c) {
-					case 'a': {
-						float alfa_c;
-
-						sscanf(aaa, "%f", &alfa_c);
-						if (alfa_c > 1 || alfa_c < 0) {
-							fprintf(stderr,
-								"++++ Bad parameter for flag -a: %s\n",
-								aaa);
-							return 2;
-						}
-						cfmt.maxshrink = alfa_c;
-						lock_fmt(&cfmt.maxshrink);
+					case 'a':
+						interpret_fmt_line("maxshrink",
+								aaa, 1);
 						break;
-					    }
 					case 'B':
-						sscanf(aaa, "%d", &cfmt.barsperstaff);
-						lock_fmt(&cfmt.barsperstaff);
+						interpret_fmt_line("barsperstaff",
+								aaa, 1);
 						break;
 					case 'b':
-						sscanf(aaa, "%d", &cfmt.measurefirst);
-						lock_fmt(&cfmt.measurefirst);
+						interpret_fmt_line("measurefirst",
+								aaa, 1);
 						break;
 					case 'D':
 						styd = aaa;
 						break;
 					case 'd':
-						cfmt.staffsep = scan_u(aaa);
-						lock_fmt(&cfmt.staffsep);
+						interpret_fmt_line("staffsep",
+								aaa, 1);
 						break;
 					case 'F':
 						read_def_format();
@@ -420,8 +434,8 @@ int main(int argc,
 						}
 						break;
 					case 'I':
-						cfmt.indent = scan_u(aaa);
-						lock_fmt(&cfmt.indent);
+						interpret_fmt_line("indent",
+								aaa, 1);
 						break;
 					case 'j':
 					case 'k':
@@ -433,18 +447,12 @@ int main(int argc,
 						lock_fmt(&cfmt.measurebox);
 						break;
 					case 'L':
-						sscanf(aaa, "%d", &cfmt.encoding);
-						if ((unsigned) cfmt.encoding > MAXENC) {
-							fprintf(stderr,
-								"++++ Bad encoding value %s - reset to 0\n",
-								aaa);
-							cfmt.encoding = 0;
-						}
-						lock_fmt(&cfmt.encoding);
+						interpret_fmt_line("encoding",
+								aaa, 1);
 						break;
 					case 'm':
-						cfmt.leftmargin = scan_u(aaa);
-						lock_fmt(&cfmt.leftmargin);
+						interpret_fmt_line("leftmargin",
+								aaa, 1);
 						break;
 					case 'N':
 						sscanf(aaa, "%d", &pagenumbers);
@@ -459,21 +467,20 @@ int main(int argc,
 						strcpy(outfn, aaa);
 						break;
 					case 's':
-						sscanf(aaa, "%f", &cfmt.scale);
-						lock_fmt(&cfmt.scale);
+						interpret_fmt_line("scale",
+								aaa, 1);
 						break;
-					case 'w': {
-						float w;
+					case 'T': {
+						struct cmdtblt_s *cmdtblt;
 
-						w = scan_u(aaa);
-						cfmt.rightmargin = cfmt.pagewidth - w - cfmt.leftmargin;
-						if (cfmt.rightmargin < 0)
-							fprintf(stderr, "Warning: staffwidth too big\n");
-						lock_fmt(&cfmt.rightmargin);
+						cmdtblt = cmdtblt_parse(aaa);
+						if (cmdtblt != 0)
+							cmdtblt->active = 1;
 						break;
 					    }
-					case 'W':
-						whistle_parse(aaa);
+					case 'w':
+						interpret_fmt_line("staffwidth",
+								aaa, 1);
 						break;
 					}
 					break;
@@ -567,7 +574,7 @@ static void do_filter(struct abctune *t, char *sel)
 			sel++;
 			end_sel = (int) ((unsigned) (~0) >> 1);
 			if (sscanf(sel, "%d%n", &end_sel, &n) != 1)
-				end_sel = 65000;
+				end_sel = (unsigned) ~0 >> 1;
 			else	sel += n;
 		} else	end_sel = cur_sel;
 		do_select(t, cur_sel, end_sel);
@@ -590,19 +597,19 @@ static void do_select(struct abctune *t,
 		for (s = t->first_sym; s != 0; s = s->next) {
 			if (s->type == ABC_T_INFO
 			    && s->text[0] == 'X') {
-				sscanf(s->text, "X:%d", &i);
+				if (sscanf(s->text, "X:%d", &i) != 1)
+					i = 0;
 				break;
 			}
 		}
 		print_tune = i >= first_tune && i <= last_tune;
 		if (print_tune
 		    || !t->client_data) {	/* (parse the global symbols) */
-			clrarena(2);
+			lvlarena(2);
 			do_tune(t, !print_tune);
+			clrarena(2);
 			t->client_data = (void *) 1;	/* treated */
 		}
-		if (i >= last_tune)
-			break;
 		t = t->next;
 	}
 }
@@ -734,7 +741,7 @@ static void usage(void)
 		"     -k n[b] same as '-j' (abc2ps compatibility)\n"
 		"     -b n    set the first measure number to n\n"
 		"     -f      have flat beams when bagpipe tunes\n"
-		"     -W vk   output a whistle tablature for voice 'v', key 'k'\n"
+		"     -T n[v]   output the tablature 'n' for voice 'v' / all voices\n"
 		"  .line breaks:\n"
 		"     -c      auto line break\n"
 		"     -B n    break every n bars\n"
@@ -780,47 +787,29 @@ static void wherefmtdir(void)
 }
 #endif
 
-/* -- parse the whistle command ('-Wvk') -- */
-static void whistle_parse(char *p)
+/* -- parse the tablature command ('-T n[v]') -- */
+static struct cmdtblt_s *cmdtblt_parse(char *p)
 {
-	short pitch;
-	char *q;
-	static char notes_tb[14] = "CDEFGABcdefgab";
-	static char pitch_tb[14] = {60, 62, 64, 65, 67, 69, 71,
-				    72, 74, 76, 77, 79, 81, 83};
+	struct cmdtblt_s *cmdtblt;
+	short val;
 
-	if (nwhistle >= MAXWHISTLE) {
-		fprintf(stderr, "++++ Too many '-W'\n");
-		return;
+	if (ncmdtblt >= MAXCMDTBLT) {
+		fprintf(stderr, "++++ Too many '-T'\n");
+		return 0;
 	}
-	if ((unsigned) (*p - '0') > 9) {
-		fprintf(stderr, "++++ Bad voice number in '-W'\n");
-		return;
+	if (*p == '\0')
+		val = -1;
+	else {
+		val = *p++ - '0' - 1;
+		if ((unsigned) val > MAXTBLT) {
+			fprintf(stderr, "++++ Bad tablature number in '-T'\n");
+			return 0;
+		}
 	}
-	whistle_tb[nwhistle].voice = *p++ - '0' - 1;
-	pitch = 0;
-	if (*p == '^' || *p == '_') {
-		if (*p++ == '^')
-			pitch++;
-		else	pitch--;
-	}
-	if (*p == '\0' || (q = strchr(notes_tb, *p)) == 0) {
-		fprintf(stderr, "++++ Bad note name in '-W'\n");
-		return;
-	}
-	pitch += pitch_tb[q - notes_tb];
-	p++;
-	if (*p == '#' || *p == 'b') {
-		if (*p++ == '#')
-			pitch++;
-		else	pitch--;
-	}
-	while (*p == '\'' || *p == ',') {
-		if (*p++ == '\'')
-			pitch += 12;
-		else	pitch -= 12;
-	}
-	whistle_tb[nwhistle++].pitch = pitch;
+	cmdtblt = &cmdtblts[ncmdtblt++];
+	cmdtblt->index = val;
+	cmdtblt->vn = p;
+	return cmdtblt;
 }
 
 /* -- write the program version -- */
@@ -856,9 +845,13 @@ void clrarena(int level)
 	a_p->r = sizeof a_p->str;
 }
 
-void lvlarena(int level)
+int lvlarena(int level)
 {
+	int old_level;
+
+	old_level = str_level;
 	str_level = level;
+	return old_level;
 }
 
 /* The area is 8 bytes aligned to handle correctly int and pointers access
