@@ -51,7 +51,6 @@
 #define GSTEM_XOFF	1.6	/* x offset for grace note stem */
 
 #define BETA_C		0.1	/* max expansion for flag -c */
-#define ALFA_X		1.0	/* max compression before complaining */
 #define BETA_X		1.0	/* max expansion before complaining */
 
 #define VOCPRE		0.4	/* portion of vocals word before note */
@@ -118,8 +117,8 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 	struct SYMBOL *next, *prev;	/* voice linkage */
 	unsigned char type;	/* symbol type */
 #define NO_TYPE		0	/* invalid type */
-#define NOTE		1	/* valid symbol types */
-#define REST		2
+#define NOTEREST	1	/* valid symbol types */
+#define SPACE		2
 #define BAR		3
 #define CLEF		4
 #define TIMESIG 	5
@@ -131,7 +130,7 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 #define GRACE		11
 #define FMTCHG		12
 #define TUPLET		13
-#define WHISTLE		14
+#define STBRK		14
 	unsigned char voice;	/* voice (0..nvoice) */
 	unsigned char staff;	/* staff (0..nstaff) */
 	unsigned char nhd;	/* number of notes in chord - 1 */
@@ -141,7 +140,7 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 	int time;		/* starting time */
 	unsigned int sflags;	/* symbol flags */
 #define S_EOLN		0x0001		/* end of line */
-#define S_WORD_ST	0x0002		/* word starts here */
+#define S_BEAM_ST	0x0002		/* beam starts here */
 #define S_BEAM_BR1	0x0004		/* 2nd beam must restart here */
 #define S_OTHER_HEAD	0x0008		/* don't draw any note head */
 #define S_IN_TUPLET	0x0010		/* in a tuplet */
@@ -159,6 +158,10 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 #define S_REPEAT	0x00020000	/* sequence / measure repeat */
 #define S_NL		0x00040000	/* start of new music line */
 #define S_SEQST		0x00080000	/* start of vertical sequence */
+#define S_SECOND	0x00100000	/* symbol on a secondary voice */
+#define S_FLOATING	0x00200000	/* symbol on a floating voice */
+#define S_NOREPBRA	0x00400000	/* don't print the repeat bracket */
+#define S_BEAM_END	0x00800000	/* beam starts here */
 	signed char stem;	/* 1 / -1 for stem up / down */
 	signed char nflags;	/* number of note flags when > 0 */
 	char dots;		/* number of dots */
@@ -177,18 +180,16 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 				 *	- BAR: new bar number
 				 *	- TUPLET: tuplet format
 				 *	- FMTCHG (format change): subtype */
-#define STBRK 0				/* staff break
-					 *	xmx: width
-					 *	doty: force if != 0 */
-#define PSSEQ 1				/* postscript sequence */
-#define REPEAT 2			/* repeat sequence or measure
+#define PSSEQ 0				/* postscript sequence */
+#define REPEAT 1			/* repeat sequence or measure
 					 *	doty: # measures if > 0
 					 *	      # notes/rests if < 0
 					 *	nohdix: # repeat */
 	float x;		/* x offset */
 	signed char y;		/* y offset of note head */
 	signed char ymn, ymx, yav; /* min, max, avg note head y offset */
-	float xmx;		/* max h-pos of a head rel to top */
+	float xmx;		/* max h-pos of a head rel to top
+				 * width when STBRK */
 	float xs, ys;		/* offset of stem end */
 	float wl, wr;		/* left, right min width */
 	float space;		/* natural space before symbol */
@@ -197,8 +198,9 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 	float shhd[MAXHD];	/* horizontal shift for heads */
 	float shac[MAXHD];	/* horizontal shift for accidentals */
 	struct lyrics *ly;	/* lyrics */
-	struct SYMBOL *grace;	/* grace notes */
-	signed char doty;	/* dot y pos when voices overlap */
+	struct SYMBOL *extra;	/* extra symbols (grace notes, tempo... */
+	signed char doty;	/* dot y pos when voices overlap
+				 * forced when STBRK */
 };
 
 /* bar types !tied to abcparse.h! */
@@ -222,7 +224,7 @@ struct FORMAT { 		/* struct for page layout */
 	float indent, infospace, slurheight, notespacingfactor;
 	float maxstaffsep, maxsysstaffsep, stemheight;
 	int abc2pscompat, alignbars, aligncomposer, autoclef;
-	int barsperstaff, breathlow, bstemdown, comball;
+	int barsperstaff, breakoneoln, bstemdown, comball;
 	int combinevoices, contbarnb, continueall, dynalign;
 	int encoding, exprabove, exprbelow, flatbeams, freegchord;
 	int infoline, gchordbox, graceslurs, gracespace, hyphencont;
@@ -294,27 +296,37 @@ extern time_t mtime;		/* last modification time of the input file */
 extern int file_initialized;	/* for output file */
 extern FILE *fout;		/* output file */
 
-#define MAXWHISTLE	4	/* max number of whistle tablature */
-struct WHISTLE_S {
-	short voice;		/* voice number */
-	short pitch;		/* absolute key pitch */
+#define MAXTBLT 8
+struct tblt_s {
+	char *head;		/* PS head function */
+	char *note;		/* PS note function */
+	char *bar;		/* PS bar function */
+	float wh;		/* width of head */
+	float ha;		/* height above the staff */
+	float hu;		/* height under the staff */
+	short pitch;		/* pitch when no associated 'w:' / 0 */
+	char instr[2];		/* instrument pitch */
 };
-extern struct WHISTLE_S whistle_tb[MAXWHISTLE];
-extern int nwhistle;
+extern struct tblt_s *tblts[MAXTBLT];
+
+#define MAXCMDTBLT	4	/* max number of -T in command line */
+struct cmdtblt_s {
+	short index;		/* tablature number */
+	short active;		/* activate or not */
+	char *vn;		/* voice name */
+};
+extern struct cmdtblt_s cmdtblts[MAXCMDTBLT];
+extern int ncmdtblt;
 
 extern int s_argc;		/* command line arguments */
 extern char **s_argv;
 
 struct STAFF_S {
 	struct clef_s clef;	/* base clef */
-	char flags[2];		/* brace and bracket flags (from %%staves) */
 	char forced_clef;	/* explicit clef */
 	char empty;		/* no symbol on this staff */
 	short botbar, topbar;	/* bottom and top of bar */
 	float y;		/* y position */
-	float bar_height;	/* height of measure bars */
-	float sep;		/* distance to the next staff */
-	float maxsep;		/* max distance to the next staff */
 	float top[YSTEP], bot[YSTEP];	/* top/bottom y offsets */
 };
 extern struct STAFF_S staff_tb[MAXSTAFF];
@@ -322,39 +334,32 @@ extern int nstaff;		/* (0..MAXSTAFF-1) */
 
 struct VOICE_S {
 	struct SYMBOL *sym;	/* associated symbols */
-	struct SYMBOL *last_symbol;	/* last symbol while scanning */
-	struct SYMBOL *s_anc;	/* ancillary symbol pointer */
-	struct VOICE_S *next, *prev;	/* staff links */
+	struct SYMBOL *last_sym;	/* last symbol while scanning */
+	struct VOICE_S *next;	/* link */
 	char *name;		/* voice id */
 	char *nm;		/* voice name */
 	char *snm;		/* voice subname */
 	char *bar_text;		/* bar text at start of staff when bar_start */
 	struct SYMBOL *tie;	/* note with ties of previous line */
 	struct SYMBOL *rtie;	/* note with ties before 1st repeat bar */
-	char *tabhead;		/* tablature:	PS head function */
-	char *tabnote;		/*		note function */
-	char *tabbar;		/*		bar function */
-	float tabha;		/*		height above the staff */
-	float tabhu;		/*		height under the staff */
+	struct tblt_s *tblts[2]; /* tablatures */
 	float scale;		/* scale */
-	float sep;		/* distance to the next staff */
-	float maxsep;		/* max distance to the next staff */
 	int time;		/* current time while parsing */
 	struct clef_s clef;	/* current clef */
 	struct key_s key;	/* current key signature */
 	struct meter_s meter;	/* current time signature */
 	unsigned hy_st;		/* lyrics hyphens at start of line (bit array) */
+	unsigned ignore:1;	/* ignore this voice */
 	unsigned forced_clef:1;	/* explicit clef */
 	unsigned second:1;	/* secondary voice in a brace/parenthesis */
-	unsigned floating:1;	/* floating voice in a brace */
-	unsigned selected:1;	/* selected while sorting by time (music.c) */
+	unsigned floating:1;	/* floating voice in a brace system */
 	unsigned bar_repeat:1;	/* bar at start of staff is a repeat bar */
 	unsigned norepbra:1;	/* don't display the repeat brackets */
 	unsigned have_ly:1;	/* some lyrics in this voice */
-	unsigned whistle:1;	/* tin whistle for this voice */
 	unsigned new_name:1;	/* redisplay the voice name */
+	unsigned space:1;	/* have a space before the next note (parsing) */
 	short wmeasure;		/* measure duration while parsing */
-	signed char bar_start;	/* bar type at start of staff / 0 */
+	short bar_start;	/* bar type at start of staff / 0 */
 	signed char clone;	/* duplicate from this voice number */
 	unsigned char staff;	/* staff (0..n-1) */
 	unsigned char cstaff;	/* staff while parsing */
@@ -368,6 +373,7 @@ struct VOICE_S {
 extern struct VOICE_S voice_tb[MAXVOICE]; /* voice table */
 extern struct VOICE_S *first_voice; /* first_voice */
 
+extern struct SYMBOL *tsfirst;	/* first symbol in the time linked list */
 extern struct SYMBOL *tsnext;	/* next line when cut */
 extern float realwidth;		/* real staff width while generating */
 
@@ -375,6 +381,27 @@ extern float realwidth;		/* real staff width while generating */
 #define C_XFLAGS 5		/* index of crotchet in flags tables */
 extern float space_tb[NFLAGS_SZ]; /* note spacing */
 
+struct SYSTEM {			/* staff system */
+	struct SYSTEM *next;
+	struct {
+		short flags;	/* brace and bracket flags (from %%staves) */
+		char empty;
+		char dum;
+		struct clef_s clef;
+		float sep, maxsep;
+	} staff[MAXSTAFF];
+	int nstaff;
+	struct {
+		signed char range;
+		unsigned char staff;
+		char second;
+		char dum;
+		float sep, maxsep;
+		struct clef_s clef;
+	} voice[MAXVOICE];
+};
+struct SYSTEM *cursys;		/* current staff system */
+
 /* PUTn: add to buffer with n arguments */
 #define PUT0(f) do {sprintf(mbf,f); a2b(); } while (0)
 #define PUT1(f,a) do {sprintf(mbf,f,a); a2b(); } while (0)
@@ -386,7 +413,7 @@ extern float space_tb[NFLAGS_SZ]; /* note spacing */
 /* -- external routines -- */
 /* abc2ps.c */
 void clrarena(int level);
-void lvlarena(int level);
+int lvlarena(int level);
 char *getarena(int len);
 void strext(char *fid, char *ext);
 /* buffer.c */
@@ -404,7 +431,7 @@ void open_output_file(void);
 void write_eps(void);
 /* deco.c */
 void deco_add(char *text);
-void deco_cnv(struct deco *dc, struct SYMBOL *s);
+void deco_cnv(struct deco *dc, struct SYMBOL *s, struct SYMBOL *prev);
 unsigned char deco_intern(unsigned char deco);
 void deco_update(struct SYMBOL *s, float dx);
 float deco_width(struct SYMBOL *s);
@@ -437,9 +464,9 @@ void y_set(struct SYMBOL *s,
 /* draw.c */
 void draw_sym_near(void);
 void draw_all_symb(void);
-void draw_vname(float indent);
-void draw_whistle(void);
+float draw_systems(float indent);
 void set_scale(int staff);
+void output_ps(struct SYMBOL *s, int state);
 void putf(float f);
 void putx(float x);
 void puty(float y);
@@ -457,12 +484,12 @@ FILE *open_file(char *fn,
 void print_format(void);
 int read_fmt_file(char *filename);
 void set_format(void);
+struct tblt_s *tblt_parse(char *p);
 /* music.c */
 void output_music(void);
 void reset_gen(void);
 /* parse.c */
 extern float multicol_start;
-void voice_dup(void);
 void do_tune(struct abctune *t,
 	     int header_only);
 void identify_note(struct SYMBOL *s,
