@@ -1,11 +1,9 @@
 /*++
  * Generic ABC parser.
  *
- * Copyright (C) 1998-2007 Jean-Franois Moine
+ * Copyright (C) 1998-2008 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996, 1997  Michael Methfessel
  *
- * Original site: http://moinejf.free.fr/
- *      
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -66,8 +64,6 @@ static struct {			/* voice table and current pointer */
 	char name[32];			/* voice name */
 	struct abcsym *last_note;	/* last note or rest */
 	short ulen;			/* unit note length */
-	unsigned char slur;		/* slurs starting on the note */
-	char dotted_slur, dotted_tie;	/* dotted slurs / ties */
 	signed char add_pitch;		/* key transpose */
 	unsigned char mvoice;		/* main voice when voice overlay */
 } voice_tb[MAXVOICE], *curvoice;
@@ -78,7 +74,7 @@ static struct {			/* voice table and current pointer */
 #define CHAR_NOTE 2
 #define CHAR_REST 3
 #define CHAR_ACC 4
-#define CHAR_GRACE 5
+#define CHAR_GR_ST 5
 #define CHAR_DECO 6
 #define CHAR_GCHORD 7
 #define CHAR_BSLASH 8
@@ -92,6 +88,7 @@ static struct {			/* voice table and current pointer */
 #define CHAR_BRHY 16
 #define CHAR_DECOS 17
 #define CHAR_SLASH 18
+#define CHAR_GR_EN 19
 static char char_tb[256] = {
 	0, 0, 0, 0, 0, 0, 0, 0, 0, CHAR_SPAC, 0, 0, 0, 0, 0, 0,	/* 00 - 0f */
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/* 10 - 1f */
@@ -117,8 +114,8 @@ static char char_tb[256] = {
 	CHAR_DECO, CHAR_DECO, CHAR_DECO, CHAR_DECO, 	/* l m n o */
 	CHAR_DECO, CHAR_DECO, CHAR_DECO, CHAR_DECO, 	/* p q r s */
 	CHAR_DECO, CHAR_DECO, CHAR_DECO, CHAR_DECO, 	/* t u v w */
-	CHAR_REST, CHAR_REST, CHAR_REST, CHAR_GRACE, 	/* x y z { */
-	CHAR_BAR, CHAR_BAD, CHAR_DECO, CHAR_BAD, 	/* | } ~ (del) */
+	CHAR_REST, CHAR_REST, CHAR_REST, CHAR_GR_ST, 	/* x y z { */
+	CHAR_BAR, CHAR_GR_EN, CHAR_DECO, CHAR_BAD, 	/* | } ~ (del) */
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/* 80 - 8f */
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/* 90 - 9f */
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/* a0 - af */
@@ -147,7 +144,8 @@ static char *parse_gchord(char *p,
 static int parse_line(struct abctune *t,
 		      char *p);
 static char *parse_note(struct abctune *t,
-			char *p);
+			char *p,
+			int flags);
 static void syntax(char *msg, char *q);
 static void vover_new(void);
 
@@ -720,7 +718,7 @@ static void parse_clef(struct abcsym *s,
 
 	s->u.clef.type = clef;
 	s->u.clef.line = clef_line;
-	s->u.clef.transpose = curvoice->add_pitch = transpose;
+	curvoice->add_pitch = transpose;
 	s->u.clef.stafflines = -1;
 	s->u.clef.staffscale = 0;
 	if (lines != 0) {
@@ -908,7 +906,7 @@ static void parse_key(char *p,
 			int i;
 
 			for (i = MAXVOICE; --i >= 0; )
-				voice_tb[i].add_pitch = s->u.clef.transpose;
+				voice_tb[i].add_pitch = curvoice->add_pitch;
 		}
 	}
 }
@@ -1161,12 +1159,13 @@ static void parse_staves(char *p,
 			 int score)
 {
 	int voice, flags_st, brace, bracket, parenth;
-	char flags[2];
+	short flags;
 	struct staff_s *p_staff;
+	char *err;
 
 	/* define the voices */
-	flags[0] = score ? STOP_BAR : 0;
-	flags[1] = 0;
+	err = 0;
+	flags = score ? STOP_BAR : 0;
 	brace = bracket = parenth = 0;
 	flags_st = 0;
 	voice = 0;
@@ -1176,39 +1175,55 @@ static void parse_staves(char *p,
 		case '\t':
 			break;
 		case '[':
-			if (parenth || brace + bracket >= 2)
-				goto err;
-			flags[brace + bracket] |= OPEN_BRACKET;
+			if (parenth || brace + bracket >= 2) {
+				if (!err)
+					err = p;
+				break;
+			}
+			if (brace + bracket == 0)
+				flags |= OPEN_BRACKET;
+			else	flags |= OPEN_BRACKET2;
 			bracket++;
 			flags_st <<= 8;
 			flags_st |= OPEN_BRACKET;
 			break;
 		case '{':
-			if (parenth || brace || bracket >= 2)
-				goto err;
-			flags[bracket] |= OPEN_BRACE;
+			if (parenth || brace || bracket >= 2) {
+				if (!err)
+					err = p;
+				break;
+			}
+			if (bracket == 0)
+				flags |= OPEN_BRACE;
+			else	flags |= OPEN_BRACE2;
 			brace++;
 			flags_st <<= 8;
 			flags_st |= OPEN_BRACE;
 			break;
 		case '(':
-			if (parenth)
-				goto err;
-			flags[0] |= OPEN_PARENTH;
+			if (parenth) {
+				if (!err)
+					err = p;
+				break;
+			}
+			flags |= OPEN_PARENTH;
 			parenth++;
 			flags_st <<= 8;
 			flags_st |= OPEN_PARENTH;
 			break;
 		case '*':
-			flags[0] |= FL_VOICE;
-			p++;
+			flags |= FL_VOICE;
 			break;
 		default:
-			if (!isalnum((unsigned) *p) && *p != '_')
-				goto err;
+			if (!isalnum((unsigned) *p) && *p != '_') {
+				if (!err)
+					err = p;
+				break;
+			}
 			if (voice >= MAXVOICE) {
-				syntax("Too many voices in %%staves", p);
-				return;
+				if (!err)
+					err = p;
+				break;
 			}
 			{
 				int v;
@@ -1224,42 +1239,53 @@ static void parse_staves(char *p,
 					p++;
 					continue;
 				case ']':
-					if (!(flags_st & OPEN_BRACKET))
-						goto err;
+					if (!(flags_st & OPEN_BRACKET)) {
+						if (!err)
+							err = p;
+						break;
+					}
 					bracket--;
-					flags[brace + bracket] |= CLOSE_BRACKET;
+					if (brace + bracket == 0)
+						flags |= CLOSE_BRACKET;
+					else	flags |= CLOSE_BRACKET2;
 					flags_st >>= 8;
 					p++;
 					continue;
 				case '}':
-					if (!(flags_st & OPEN_BRACE))
-						goto err;
+					if (!(flags_st & OPEN_BRACE)) {
+						if (!err)
+							err = p;
+						break;
+					}
 					brace--;
-					flags[bracket] |= CLOSE_BRACE;
+					if (bracket == 0)
+						flags |= CLOSE_BRACE;
+					else	flags |= CLOSE_BRACE2;
 					flags_st >>= 8;
 					p++;
 					continue;
 				case ')':
-					if (!(flags_st & OPEN_PARENTH))
-						goto err;
+					if (!(flags_st & OPEN_PARENTH)) {
+						if (!err)
+							err = p;
+						break;
+					}
 					parenth--;
-					flags[0] |= CLOSE_PARENTH;
+					flags |= CLOSE_PARENTH;
 					flags_st >>= 8;
 					p++;
 					continue;
 				case '|':
 					if (!score)
-						flags[0] |= STOP_BAR;
-					else	flags[0] &= ~STOP_BAR;
+						flags |= STOP_BAR;
+					else	flags &= ~STOP_BAR;
 					p++;
 					continue;
 				}
 				break;
 			}
-			p_staff->flags[0] = flags[0];
-			p_staff->flags[1] = flags[1];
-			flags[0] = score ? STOP_BAR : 0;
-			flags[1] = 0;
+			p_staff->flags = flags;
+			flags = score ? STOP_BAR : 0;
 			if (*p == '\0')
 				break;
 			continue;
@@ -1268,23 +1294,18 @@ static void parse_staves(char *p,
 			break;
 		p++;
 	}
-	if (flags_st == 0)
-		goto ok;
-err:
-	syntax("%%staves error", p);
-	{
+	if (flags_st != 0 || err) {
 		int i;
 
-		for (i = 0; i < voice; i++) {
-			s->u.staves[i].flags[0] &= ~(OPEN_PARENTH | CLOSE_PARENTH
+		syntax("%%staves error", err);
+		for (i = 0; i < voice; i++)
+			s->u.staves[i].flags &= ~(OPEN_PARENTH | CLOSE_PARENTH
 						| OPEN_BRACKET | CLOSE_BRACKET
-						| OPEN_BRACE | OPEN_BRACE);
-			s->u.staves[i].flags[1] &= ~(OPEN_BRACKET | CLOSE_BRACKET
-						| OPEN_BRACE | OPEN_BRACE);
-		}
+						| OPEN_BRACE | OPEN_BRACE
+						| OPEN_BRACKET2 | CLOSE_BRACKET2
+						| OPEN_BRACE2 | OPEN_BRACE2);
+		s->flags |= ABC_F_ERROR;
 	}
-	s->flags |= ABC_F_ERROR;
-ok:
 	if (voice < MAXVOICE)
 		s->u.staves[voice].voice = -1;
 }
@@ -1983,8 +2004,7 @@ static int parse_line(struct abctune *t,
 	char *comment, *q, c;
 	struct abcsym *last_note_sav = 0;
 	struct deco dc_sav;
-	int i;
-	char sappo = 0;
+	int i, flags, flags_sav = 0, slur;
 	static char qtb[10] = {0, 1, 3, 2, 3, 0, 2, 0, 3, 0};
 
 	colnum = 0;
@@ -2119,17 +2139,21 @@ again:					/* for history */
 		return 0;
 	}
 
-	if (isspace((unsigned) scratch_line[0]))
-		t->last_sym->flags |= ABC_F_WORD_END;
+	flags = (isspace((unsigned) scratch_line[0]))
+		? ABC_F_SPACE
+		: 0;
 
 	lyric_started = 0;
 	deco_start = deco_cont = 0;
+	slur = 0;
 	while (*p != '\0') {
 		colnum = p - scratch_line;
 		switch (char_tb[(unsigned) *p++]) {
 		case CHAR_GCHORD: {			/* " */
 			int more_gch;
 
+			if (flags & ABC_F_GRACE)
+				goto bad_char;
 			for (;;) {
 				p = parse_gchord(p, &more_gch);
 				if (!more_gch)
@@ -2142,27 +2166,28 @@ again:					/* for history */
 			}
 			break;
 		    }
-		case CHAR_GRACE:		/* '{' or '}' */
-			if (p[-1] == '{') {
-				if (*p == '/') {
-					sappo = 1;
-					p++;
-				}
-				char_tb['{'] = CHAR_BAD;
-				char_tb['}'] = CHAR_GRACE;
-				last_note_sav = curvoice->last_note;
-				curvoice->last_note = 0;
-				memcpy(&dc_sav, &dc, sizeof dc);
-				dc.n = dc.h = dc.s = 0;
-			} else {
-				char_tb['{'] = CHAR_GRACE;
-				char_tb['}'] = CHAR_BAD;
-/*fixme:bad*/
-				t->last_sym->flags |= (ABC_F_WORD_END | ABC_F_GR_END);
-				curvoice->last_note = last_note_sav;
-				memcpy(&dc, &dc_sav, sizeof dc);
+		case CHAR_GR_ST:		/* '{' */
+			if (flags & ABC_F_GRACE)
+				goto bad_char;
+			last_note_sav = curvoice->last_note;
+			curvoice->last_note = 0;
+			memcpy(&dc_sav, &dc, sizeof dc);
+			dc.n = dc.h = dc.s = 0;
+			flags_sav = flags;
+			flags = ABC_F_GRACE;
+			if (*p == '/') {
+				flags |= ABC_F_SAPPO;
+				p++;
 			}
 			break;
+		case CHAR_GR_EN:		/* '}' */
+			if (!(flags & ABC_F_GRACE))
+				goto bad_char;
+			t->last_sym->flags |= ABC_F_GR_END;
+			curvoice->last_note = last_note_sav;
+			memcpy(&dc, &dc_sav, sizeof dc);
+			flags = flags_sav;
+			break;
 		case CHAR_DECO:
 		case CHAR_DECOS:
 			if (p[-1] == '!' && check_nl(p)) {
@@ -2173,11 +2198,11 @@ again:					/* for history */
 			}
 			if (p[-1] == '.') {
 				if (*p == '(') {
-					curvoice->dotted_slur = 1;
+					flags |= ABC_F_DOTTED_SLUR;
 					break;
 				}
 				if (*p == '-') {
-					curvoice->dotted_tie = 1;
+					flags |= ABC_F_DOTTED_TIE;
 					break;
 				}
 				if (*p == '|') {
@@ -2194,15 +2219,16 @@ again:					/* for history */
 		case CHAR_ACC:
 		case CHAR_NOTE:
 		case CHAR_REST:
-			p = parse_note(t, p - 1);
-			if (sappo) {
-				t->last_sym->flags |= ABC_F_SAPPO;
-				sappo = 0;
-			}
+			p = parse_note(t, p - 1, flags);
+			flags &= ABC_F_GRACE;
+			t->last_sym->u.note.slur_st = slur;
+			slur = 0;
 			if (t->last_sym->u.note.lens[0] > 0)	/* if not space */
 				curvoice->last_note = t->last_sym;
 			break;
 		case CHAR_SLASH:		/* '/' */
+			if (flags & ABC_F_GRACE)
+				goto bad_char;
 			q = p;
 			while (*q == '/')
 				q++;
@@ -2217,32 +2243,30 @@ again:					/* for history */
 			p = q;
 			break;
 		case CHAR_BSLASH:		/* '\\' */
-/*fixme: KO if in grace note sequence*/
-			if (*p == '\0') {
-				s = abc_new(t, 0, 0);	/* abc2win EOL */
-				s->type = ABC_T_EOLN;
-				s->u.eoln.type = 1;
-				return 0;
-			}
+			if (*p == '\0')
+				break;
 			syntax("'\\' ignored", p - 1);
 			break;
 		case CHAR_OBRA:			/* '[' */
 			if (*p == '|' || *p == ']'
 			    || isdigit((unsigned) *p) || *p == '"') {
+				if (flags & ABC_F_GRACE)
+					goto bad_char;
 				p = parse_bar(t, p);
 				break;
 			}
 			if (p[1] != ':') {
-				p = parse_note(t, p - 1);	/* chord */
-				if (sappo) {
-					t->last_sym->flags |= ABC_F_SAPPO;
-					sappo = 0;
-				}
+				p = parse_note(t, p - 1, flags); /* chord */
+				flags &= ABC_F_GRACE;
+				t->last_sym->u.note.slur_st = slur;
+				slur = 0;
 				curvoice->last_note = t->last_sym;
 				break;
 			}
 
 			/* embedded header */
+			if (flags & ABC_F_GRACE)
+				goto bad_char;
 			c = ']';
 			q = p;
 			while (*p != '\0' && *p != c)
@@ -2259,6 +2283,8 @@ again:					/* for history */
 				p++;
 			break;
 		case CHAR_BAR:			/* '|', ':' or ']' */
+			if (flags & ABC_F_GRACE)
+				goto bad_char;
 			p = parse_bar(t, p);
 			break;
 		case CHAR_OPAR:			/* '(' */
@@ -2302,11 +2328,12 @@ again:					/* for history */
 				s->u.tuplet.p_plet = pplet;
 				s->u.tuplet.q_plet = qplet;
 				s->u.tuplet.r_plet = rplet;
-				if (char_tb['{'] == CHAR_BAD)
-					s->flags |= ABC_F_GRACE;
+				s->flags |= flags;
 				break;
 			}
 			if (*p == '&') {
+				if (flags & ABC_F_GRACE)
+					goto bad_char;
 				p++;
 				if (vover != 0) {
 					syntax("Nested voice overlay", p - 1);
@@ -2319,18 +2346,18 @@ again:					/* for history */
 				vover = -1;		/* multi-bars */
 				break;
 			}
-			curvoice->slur <<= 2;
+			slur <<= 2;
 			switch (*p) {
 			case '\'':
-				curvoice->slur += SL_ABOVE;
+				slur += SL_ABOVE;
 				p++;
 				break;
 			case ',':
-				curvoice->slur += SL_BELOW;
+				slur += SL_BELOW;
 				p++;
 				break;
 			default:
-				curvoice->slur += SL_AUTO;
+				slur += SL_AUTO;
 				break;
 			}
 			break;
@@ -2345,6 +2372,8 @@ again:					/* for history */
 			t->last_sym->u.note.slur_end++;
 			break;
 		case CHAR_VOV:			/* '&' */
+			if (flags & ABC_F_GRACE)
+				goto bad_char;
 			if (*p != ')'
 			    || vover == 0) {		/*??*/
 				if (curvoice->last_note == 0) {
@@ -2370,7 +2399,7 @@ again:					/* for history */
 			curvoice = &voice_tb[curvoice->mvoice];
 			break;
 		case CHAR_SPAC:			/* ' ' and '\t' */
-			t->last_sym->flags |= ABC_F_WORD_END;
+			flags |= ABC_F_SPACE;
 			break;
 		case CHAR_MINUS: {		/* '-' */
 			int tie_pos;
@@ -2395,9 +2424,9 @@ again:					/* for history */
 				if (curvoice->last_note->u.note.ti1[i] == 0)
 					curvoice->last_note->u.note.ti1[i] = tie_pos;
 			}
-			if (curvoice->dotted_tie) {
+			if (flags & ABC_F_DOTTED_TIE) {
 				curvoice->last_note->flags |= ABC_F_DOTTED_TIE;
-				curvoice->dotted_tie = 0;
+				flags &= ~ABC_F_DOTTED_TIE;
 			}
 			break;
 		    }
@@ -2422,19 +2451,20 @@ again:					/* for history */
 			break;
 		default:
 		bad_char:
-			syntax("Bad character", p - 1);
+			syntax((flags & ABC_F_GRACE)
+					? "Bad character in grace note sequence"
+					: "Bad character",
+				p - 1);
 			break;
 		}
 	}
 
 /*fixme: may we have grace notes across lines?*/
-	if (char_tb['{'] == CHAR_BAD) {
-		syntax("No end of grace note sequence", 0);
-		char_tb['{'] = CHAR_GRACE;
-		char_tb['}'] = CHAR_BAD;
+	if (flags & ABC_F_GRACE) {
+		syntax("EOLN in grace note sequence", p - 1);
 		if (curvoice->last_note != 0)
-			curvoice->last_note->flags |=
-				(ABC_F_WORD_END | ABC_F_GR_END | ABC_F_ERROR);
+			curvoice->last_note->flags
+				|= (ABC_F_GR_END | ABC_F_ERROR);
 		curvoice->last_note = last_note_sav;
 		memcpy(&dc, &dc_sav, sizeof dc);
 	}
@@ -2442,22 +2472,26 @@ again:					/* for history */
 	/* add eoln */
 	s = abc_new(t, 0, 0);
 	s->type = ABC_T_EOLN;
-/*	s->as.u.eoln = 0; */
+	if (flags & ABC_F_SPACE)
+		s->flags |= ABC_F_SPACE;
+	if (p[-1] == '\\')
+		s->u.eoln.type = 1;
+/*	else	s->u.eoln.type = 0; */
 	return 0;
 }
 
 /* -- parse a note or a rest -- */
 static char *parse_note(struct abctune *t,
-			char *p)
+			char *p,
+			int flags)
 {
 	struct abcsym *s;
 	char *q;
 	int pit, len, acc, nostem, chord, j, m;
 
-	if (char_tb['{'] == CHAR_BAD) {	/* in a grace note sequence */
+	if (flags & ABC_F_GRACE)	/* in a grace note sequence */
 		s = abc_new(t, 0, 0);
-		s->flags |= ABC_F_GRACE;
-	} else {
+	else {
 		s = abc_new(t, gchord, 0);
 		if (gchord) {
 			if (free_f)
@@ -2466,8 +2500,9 @@ static char *parse_note(struct abctune *t,
 		}
 	}
 	s->type = ABC_T_NOTE;
+	s->flags |= flags;
 
-	if (*p != 'Z' && !lyric_started && !(s->flags & ABC_F_GRACE)) {
+	if (*p != 'Z' && !lyric_started && !(flags & ABC_F_GRACE)) {
 		lyric_started = 1;
 		s->flags |= ABC_F_LYRIC_START;
 		deco_start = s;
@@ -2494,7 +2529,7 @@ static char *parse_note(struct abctune *t,
 			s->u.note.lens[1] = strtol(p, &q, 10);
 			p = q;
 		} else	s->u.note.lens[1] = -1;
-		goto add_slurs;
+		goto add_deco;
 	case 'x':			/* invisible rest */
 		s->flags |= ABC_F_INVIS;
 		/* fall thru */
@@ -2559,7 +2594,7 @@ static char *parse_note(struct abctune *t,
 				p++;
 		} else {
 			p = parse_basic_note(p, &pit, &len, &acc, &tmp);
-			if (s->flags & ABC_F_GRACE) {
+			if (flags & ABC_F_GRACE) {
 				len = len * BASE_LEN / 4 / ulen;
 				tmp = 0;
 			}
@@ -2640,13 +2675,6 @@ do_brhythm:
 	    && curvoice->last_note->u.note.brhythm != 0)
 		broken_rhythm(&s->u.note,
 			      -curvoice->last_note->u.note.brhythm);
-add_slurs:
-	s->u.note.slur_st = curvoice->slur;
-	curvoice->slur = 0;
-	if (curvoice->dotted_slur) {
-		s->flags |= ABC_F_DOTTED_SLUR;
-		curvoice->dotted_slur = 0;
-	}
 add_deco:
 	if (dc.n > 0) {
 		memcpy(s->type != ABC_T_MREST ? &s->u.note.dc
