@@ -84,7 +84,7 @@ struct abcsym {
 	unsigned short flags;
 #define ABC_F_ERROR	0x0001		/* error around this symbol */
 #define ABC_F_INVIS	0x0002		/* invisible symbol */
-#define ABC_F_WORD_END	0x0004		/* 1 if word ends here */
+#define ABC_F_SPACE	0x0004		/* space before a note */
 #define ABC_F_STEMLESS	0x0008		/* note with no stem */
 #define ABC_F_LYRIC_START 0x0010	/* may start a lyric here */
 #define ABC_F_GRACE	0x0020		/* grace note */
@@ -155,7 +155,6 @@ struct abcsym {
 #define PERC 3
 			char line;
 			signed char octave;
-			signed char transpose;
 			char invis;
 			char check_pitch;	/* check if old abc2ps transposition */
 		} clef;
@@ -171,7 +170,7 @@ struct abcsym {
 		} eoln;
 		struct staff_s {	/* %%staves */
 			short voice;
-			unsigned char flags[2];
+			short flags;
 #define OPEN_BRACE 0x01
 #define CLOSE_BRACE 0x02
 #define OPEN_BRACKET 0x04
@@ -180,6 +179,10 @@ struct abcsym {
 #define CLOSE_PARENTH 0x20
 #define STOP_BAR 0x40
 #define FL_VOICE 0x80
+#define OPEN_BRACE2 0x0100
+#define CLOSE_BRACE2 0x0200
+#define OPEN_BRACKET2 0x0400
+#define CLOSE_BRACKET2 0x0800
 		} staves[MAXVOICE];
 		struct {		/* voice overlay */
 			char type;
