@@ -507,7 +507,7 @@ static void d_trill(struct deco_elt *de)
 	if (de->start != 0) {		/* deco start */
 		s = de->start->s;
 		x = s->x;
-		if (s->type == NOTE
+		if (s->as.type == ABC_T_NOTE
 		    && s->as.u.note.dc.n > 1)
 			x += 10;
 	} else {			/* end without start */
@@ -525,7 +525,7 @@ static void d_trill(struct deco_elt *de)
 		}
 	} else {
 		w = s2->x - x - 6;
-		if (s2->type == NOTE)
+		if (s2->as.type == ABC_T_NOTE)
 			w -= 6;
 		if (w < 20) {
 			x -= (20 - w) * 0.5;
@@ -740,7 +740,8 @@ void deco_add(char *text)
 
 /* -- convert the decorations -- */
 void deco_cnv(struct deco *dc,
-	      struct SYMBOL *s)
+	      struct SYMBOL *s,
+	      struct SYMBOL *prev)
 {
 	int i, j;
 	struct deco_def_s *dd;
@@ -772,25 +773,26 @@ void deco_cnv(struct deco *dc,
 			s->sflags |= S_BEAM_ON;
 			break;
 		case 34:		/* 34 = trem1..trem4 */
-			if (s->type != NOTE || s->prev->type != NOTE) {
+			if (s->as.type != ABC_T_NOTE
+			    || prev == 0
+			    || prev->as.type != ABC_T_NOTE) {
 				error(1, s,
 				      "+%s+ must be on the last of a couple of notes",
 				       dd->name);
 				break;
 			}
-			s->sflags |= S_TREM;
-			s->sflags &= ~S_WORD_ST;
-			s->as.flags |= ABC_F_WORD_END;
-			s->prev->sflags |= (S_TREM | S_WORD_ST);
-			s->prev->as.flags &= ~ABC_F_WORD_END;
-			s->nflags = s->prev->nflags = dd->name[4] - '0';
+			s->sflags |= (S_TREM | S_BEAM_END);
+			s->sflags &= ~S_BEAM_ST;
+			prev->sflags |= (S_TREM | S_BEAM_ST);
+			prev->sflags &= ~S_BEAM_END;
+			s->nflags = prev->nflags = dd->name[4] - '0';
 			for (j = 0; j <= s->nhd; j++)
 				s->as.u.note.lens[j] *= 2;
-			for (j = 0; j <= s->prev->nhd; j++)
-				s->prev->as.u.note.lens[j] *= 2;
+			for (j = 0; j <= prev->nhd; j++)
+				prev->as.u.note.lens[j] *= 2;
 			break;
 		case 35:		/* 35 = xstem */
-			if (s->type != NOTE) {
+			if (s->as.type != ABC_T_NOTE) {
 				error(1, s,
 				      "Cannot have +%s+ on a rest or a bar",
 				       dd->name);
@@ -799,7 +801,7 @@ void deco_cnv(struct deco *dc,
 			s->sflags |= S_XSTEM;
 			break;
 		case 36:		/* 36 = beambr1 / beambr2 */
-			if (s->type != NOTE) {
+			if (s->as.type != ABC_T_NOTE) {
 				error(1, s,
 				      "Cannot have +%s+ on a rest or a bar",
 				       dd->name);
@@ -1127,8 +1129,7 @@ static void deco_create(struct SYMBOL *s,
 		/* memorize the decorations, but not the head ones */
 		if (strncmp(dd->name, "head-", 5) == 0) {
 			switch (s->type) {
-			case NOTE:
-			case REST:
+			case NOTEREST:
 				s->sflags |= S_OTHER_HEAD;
 				break;
 			default:
@@ -1147,7 +1148,8 @@ static void deco_create(struct SYMBOL *s,
 		de->s = s;
 		de->t = dd - deco_def_tb;
 		de->staff = s->staff;
-		if (s->type == NOTE && (s->as.flags & ABC_F_GRACE))
+		if (s->as.type == ABC_T_NOTE
+		    && (s->as.flags & ABC_F_GRACE))
 			de->flags = DE_GRACE;
 		l = strlen(dd->name) - 1;
 		if (l > 0) {
@@ -1168,7 +1170,7 @@ static void deco_create(struct SYMBOL *s,
 
 		if (dd->func >= 3)	/* if not near the note */
 			continue;
-		if (s->type != NOTE) {
+		if (s->as.type != ABC_T_NOTE) {
 			error(1, s,
 			      "Cannot have +%s+ on a rest or a bar",
 			       dd->name);
@@ -1189,7 +1191,7 @@ void draw_deco_near(void)
 
 	deco_head = deco_tail = 0;
 	first = 0;
-	for (s = first_voice->sym; s != 0; s = s->ts_next) {
+	for (s = tsfirst; s != 0; s = s->ts_next) {
 		switch (s->type) {
 		case BAR:
 		case MREST:
@@ -1197,8 +1199,8 @@ void draw_deco_near(void)
 				continue;
 			dc = &s->as.u.bar.dc;
 			break;
-		case NOTE:
-		case REST:
+		case NOTEREST:
+		case SPACE:
 			if (first == 0)
 				first = s;
 			if (s->as.u.note.dc.n == 0)
@@ -1206,8 +1208,8 @@ void draw_deco_near(void)
 			dc = &s->as.u.note.dc;
 			break;
 		case GRACE:
-			for (g = s->grace; g != 0; g = g->next) {
-				if (g->type != NOTE
+			for (g = s->extra; g != 0; g = g->next) {
+				if (g->as.type != ABC_T_NOTE
 				    || g->as.u.note.dc.n == 0)
 					continue;
 				dc = &g->as.u.note.dc;
@@ -1286,14 +1288,14 @@ void draw_deco_staff(void)
 	/* search the vertical offset for the guitar chords */
 	memset(minmax, 0, sizeof minmax);
 	some_gchord = 0;
-	for (s = first_voice->sym; s != 0; s = s->ts_next) {
+	for (s = tsfirst; s != 0; s = s->ts_next) {
 		char *p;
 
 		if (s->as.text == 0)
 			continue;
 		switch (s->type) {
-		case NOTE:
-		case REST:
+		case NOTEREST:
+		case SPACE:
 		case MREST:
 			break;
 		case BAR:
@@ -1317,17 +1319,17 @@ void draw_deco_staff(void)
 	if (some_gchord) {
 		int i;
 
-		for (i = nstaff; i >= 0; i--) {
+		for (i = 0; i <= nstaff; i++) {
 			minmax[i].ymax += 4;
 			if (minmax[i].ymax < 34)
 				minmax[i].ymax = 34;
 		}
-		for (s = first_voice->sym; s != 0; s = s->ts_next) {
+		for (s = tsfirst; s != 0; s = s->ts_next) {
 			if (s->as.text == 0)
 				continue;
 			switch (s->type) {
-			case NOTE:
-			case REST:
+			case NOTEREST:
+			case SPACE:
 			case MREST:
 				break;
 			case BAR:
@@ -1346,7 +1348,7 @@ void draw_deco_staff(void)
 		float y2;
 		int i, repnl;
 
-		if (p_voice->second)
+		if (p_voice->second || p_voice->sym == 0)
 			continue;
 
 		/* search the max y offset */
@@ -1354,7 +1356,8 @@ void draw_deco_staff(void)
 		first_repeat = 0;
 		for (s = p_voice->sym->next; s != 0; s = s->next) {
 			if (s->type != BAR
-			    || !s->as.u.bar.repeat_bar)
+			    || !s->as.u.bar.repeat_bar
+			    || (s->sflags & S_NOREPBRA))
 				continue;
 /*fixme: line cut on repeat!*/
 			if (s->next == 0)
@@ -1400,7 +1403,7 @@ void draw_deco_staff(void)
 
 			/* have room for the repeat numbers */
 			if (s1->as.text != 0) {
-				w = cwid('W') * cfmt.font_tb[REPEATFONT].swfac
+				w = cwid('2') * cfmt.font_tb[REPEATFONT].swfac
 					* strlen(s1->as.text);
 				y2 = y_get(s1, 1, s1->x + 4, w, 0);
 				y2 += cfmt.font_tb[REPEATFONT].size + 2;
@@ -1417,7 +1420,8 @@ void draw_deco_staff(void)
 			char *p;
 
 			if (s->type != BAR
-			    || !s->as.u.bar.repeat_bar)
+			    || !s->as.u.bar.repeat_bar
+			    || (s->sflags & S_NOREPBRA))
 				continue;
 			s1 = s;
 			for (;;) {
@@ -1450,8 +1454,9 @@ void draw_deco_staff(void)
 				   && s2->as.u.bar.type != (B_OBRA << 4) + B_CBRA)
 				 || s2->as.u.bar.type == B_CBRA) {
 				i =  2;			/* bracket start and stop */
+/*fixme:%%staves: cursys moved?*/
 				if (s->staff > 0
-				    && !(staff_tb[s->staff - 1].flags[0] & STOP_BAR))
+				    && !(cursys->staff[s->staff - 1].flags & STOP_BAR))
 					w = s2->wl;
 				else if ((s2->as.u.bar.type & 0x0f) == B_COL)
 					w = 12;
@@ -1463,14 +1468,13 @@ void draw_deco_staff(void)
 			w = s2->x - x - w;
 			p = s1->as.text;
 			if (p == 0) {
-				i--;		/* no bracket start (1) or not draw */
+				i--;		/* no bracket start (1) or not drawn */
 				p = "";
 			}
 			if (i == 0 && s2->next == 0	/* 2nd ending at end of line */
 			    && !(s2->sflags & S_RBSTOP)) {
-				if (p_voice->bar_start != 0)
-					i = 2;		/* bracket start and stop */
-				else	repnl = 1;	/* continue on next line */
+				if (p_voice->bar_start == 0)
+					repnl = 1;	/* continue on next line */
 			}
 			if (i >= 0) {
 				PUT3("(%s)-%.1f %d ",
@@ -1650,8 +1654,7 @@ static void draw_gchord(struct SYMBOL *s,
 					switch (next->type) {
 					default:
 						continue;
-					case NOTE:
-					case REST:
+					case NOTEREST:
 					case BAR:
 						x = next->x;
 						break;
@@ -1763,17 +1766,16 @@ void draw_measnb(void)
 
 	/* get the current bar number */
 /*fixme: what to do if no symbol in the 1st voice?*/
-	if ((s = first_voice->sym->next) == 0)
+	if ((s = first_voice->sym) == 0
+	    || (s = s->next) == 0)
 		return;
 	for ( ; s->next != 0; s = s->next) {
 		switch (s->type) {
 		case TIMESIG:
 		case CLEF:
 		case KEYSIG:
-		case PART:
-		case TEMPO:
 		case FMTCHG:
-		case WHISTLE:
+		case STBRK:
 			continue;
 		case BAR:
 			if (s->u != 0)
@@ -1885,7 +1887,7 @@ void draw_measnb(void)
 		y = y_get(s, 1, x, w, 0);
 		if (y < staff_tb[0].topbar + 6)
 			y = staff_tb[0].topbar + 6;
-		if (s->next->type == NOTE) {
+		if (s->next->as.type == ABC_T_NOTE) {
 			if (s->next->stem > 0) {
 				if (y < s->next->ys - cfmt.font_tb[MEASUREFONT].size)
 					y = s->next->ys - cfmt.font_tb[MEASUREFONT].size;
@@ -2042,7 +2044,7 @@ float draw_partempo(float top,
 		    int any_part,
 		    int any_tempo)
 {
-	struct SYMBOL *s;
+	struct SYMBOL *s, *g;
 	float h, ht, w, y, ymin, dy;
 
 	/* put the tempo indication at top */
@@ -2061,9 +2063,14 @@ float draw_partempo(float top,
 		x = 0;
 /*fixme:have tempo on other voices but the 1st?*/
 		for (s = first_voice->sym->next; s != 0; s = s->next) {
-			if (s->type != TEMPO)
+			if ((g = s->extra) == 0)
 				continue;
-			w = tempo_width(s);
+			for (; g != 0; g = g->next)
+				if (g->type == TEMPO)
+					break;
+			if (g == 0)
+				continue;
+			w = tempo_width(g);
 			y = y_get(s, 1, s->x - 5, w, 0) + 2;
 			if (y > ymin)
 				ymin = y;
@@ -2083,17 +2090,21 @@ float draw_partempo(float top,
 		set_font(TEMPOFONT);
 		beat = 0;
 		for (s = first_voice->sym; s != 0; s = s->next) {
-			if (s->type != TEMPO) {
-				if (s->type == TIMESIG)
-					beat = get_beat(&s->as.u.meter);
+			if (s->type == TIMESIG)
+				beat = get_beat(&s->as.u.meter);
+			if ((g = s->extra) == 0)
+				continue;
+			for (; g != 0; g = g->next)
+				if (g->type == TEMPO)
+					break;
+			if (g == 0)
 				continue;
-			}
 
 			/*fixme: cf left shift (-5)*/
 			PUT2("%.1f %.1f M ", s->x - 5,
 				(dosh & 1) ? h : y);
 			dosh >>= 1;
-			write_tempo(s, beat, 1);
+			write_tempo(g, beat, 1);
 		}
 	} else	ht = 0;
 
@@ -2106,9 +2117,14 @@ float draw_partempo(float top,
 	str_font(PARTSFONT);
 	ymin = staff_tb[0].topbar + 14;
 	for (s = first_voice->sym->next; s != 0; s = s->next) {
-		if (s->type != PART)
+		if ((g = s->extra) == 0)
 			continue;
-		w = tex_str(&s->as.text[2]);
+		for (; g != 0; g = g->next)
+			if (g->type == PART)
+				break;
+		if (g == 0)
+			continue;
+		w = tex_str(&g->as.text[2]);
 		y = y_get(s, 1, s->x - 10, w + 15, 0) + 5;
 		if (ymin < y)
 			ymin = y;
@@ -2118,9 +2134,14 @@ float draw_partempo(float top,
 
 	set_font(PARTSFONT);
 	for (s = first_voice->sym->next; s != 0; s = s->next) {
-		if (s->type != PART)
+		if ((g = s->extra) == 0)
 			continue;
-		tex_str(&s->as.text[2]);
+		for (; g != 0; g = g->next)
+			if (g->type == PART)
+				break;
+		if (g == 0)
+			continue;
+		tex_str(&g->as.text[2]);
 		PUT4("%.1f %.1f M(%s)show%s\n",
 		     s->x - 10, 2 - ht - h,
 		     tex_buf, cfmt.partsbox ? "b" : "");
