@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2007 Jean-Franois Moine
+ * Copyright (C) 1998-2008 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -60,6 +60,7 @@ static struct format {
 	{"annotationfont", &cfmt.font_tb[ANNOTATIONFONT], FORMAT_F, 0},
 	{"barsperstaff", &cfmt.barsperstaff, FORMAT_I, 0},
 	{"botmargin", &cfmt.botmargin, FORMAT_U, 0},
+	{"breakoneoln", &cfmt.breakoneoln, FORMAT_B, 0},
 	{"bstemdown", &cfmt.bstemdown, FORMAT_B, 0},
 	{"combinevoices", &cfmt.combinevoices, FORMAT_B, 0},
 	{"comball", &cfmt.comball, FORMAT_B, 0},
@@ -320,6 +321,7 @@ void make_font_list(void)
 static void set_infoname(char *p)
 {
 	struct SYMBOL *s, *prev;
+	int old_lvl;
 
 	if (*p == 'I')
 		return;
@@ -338,6 +340,7 @@ static void set_infoname(char *p)
 		}
 		return;
 	}
+	old_lvl = lvlarena(0);
 	if (s == 0) {
 		s = (struct SYMBOL *) getarena(sizeof *s);
 		memset(s, 0, sizeof *s);
@@ -350,6 +353,7 @@ static void set_infoname(char *p)
 	}
 	s->as.text = (char *) getarena(strlen(p) + 1);
 	strcpy(s->as.text, p);
+	lvlarena(old_lvl);
 }
 
 /* -- set the default format -- */
@@ -387,6 +391,7 @@ void set_format(void)
 	f->measurenb = -1;
 	f->measurefirst = 1;
 	f->autoclef = 1;
+	f->breakoneoln = 1;
 	f->dynalign = 1;
 	f->printparts = 1;
 	f->printtempo = 1;
@@ -612,6 +617,145 @@ static void g_fspc(char *p,
 		outft = -1;
 }
 
+/* -- parse a 'tablature' definition -- */
+/* %%tablature
+ *	[#<nunmber (1..MAXTBLT)>]
+ *	[pitch=<instrument pitch (<note> # | b)>]
+ *	[[<head width>]
+ *	 <height above>]
+ *	<height under>
+ *	<head function>
+ *	<note function>
+ *	[<bar function>]
+ */
+struct tblt_s *tblt_parse(char *p)
+{
+	struct tblt_s *tblt;
+	int n;
+	char *q;
+	static char notes_tb[14] = "CDEFGABcdefgab";
+	static char pitch_tb[14] = {60, 62, 64, 65, 67, 69, 71,
+				    72, 74, 76, 77, 79, 81, 83};
+
+	/* number */
+	if (*p == '#') {
+		p++;
+		n = *p++ - '0' - 1;
+		if ((unsigned) n >= MAXTBLT
+		    || (*p != '\0' && *p != ' ')) {
+			error(1, 0, "Invalid number in %%%%tablature");
+			return 0;
+		}
+		if (*p == '\0')
+			return tblts[n];
+		while (isspace((unsigned char) *p))
+			p++;
+	} else	n = -1;
+
+	/* pitch */
+	tblt = malloc(sizeof *tblt);
+	memset(tblt, 0, sizeof *tblt);
+	if (strncmp(p, "pitch=", 6) == 0) {
+		p += 6;
+		if (*p == '^' || *p == '_') {
+			if (*p == '^') {
+				tblt->pitch++;
+				tblt->instr[1] = '#';
+			} else {
+				tblt->pitch--;
+				tblt->instr[1] = 'b';
+			}
+			p++;
+		}
+		if (*p == '\0' || (q = strchr(notes_tb, *p)) == 0) {
+			error(1, 0, "Invalid pitch in %%%%tablature");
+			return 0;
+		}
+		tblt->pitch += pitch_tb[q - notes_tb];
+		tblt->instr[0] = toupper(*p++);
+		while (*p == '\'' || *p == ',') {
+			if (*p++ == '\'')
+				tblt->pitch += 12;
+			else	tblt->pitch -= 12;
+		}
+		if (*p == '#' || *p == 'b') {
+			if (*p == '#')
+				tblt->pitch++;
+			else	tblt->pitch--;
+			tblt->instr[1] = *p++;
+		}
+		while (*p == '\'' || *p == ',') {
+			if (*p++ == '\'')
+				tblt->pitch += 12;
+			else	tblt->pitch -= 12;
+		}
+		while (isspace((unsigned char) *p))
+			p++;
+	}
+
+	/* width and heights */
+	if (!isdigit(*p)) {
+		error(1, 0, "Invalid width/height in %%%%tablature");
+		return 0;
+	}
+	tblt->hu = scan_u(p);
+	while (*p != '\0' && !isspace((unsigned char) *p))
+		p++;
+	while (isspace((unsigned char) *p))
+		p++;
+	if (isdigit(*p)) {
+		tblt->ha = tblt->hu;
+		tblt->hu = scan_u(p);
+		while (*p != '\0' && !isspace((unsigned char) *p))
+			p++;
+		while (isspace((unsigned char) *p))
+			p++;
+		if (isdigit(*p)) {
+			tblt->wh = tblt->ha;
+			tblt->ha = tblt->hu;
+			tblt->hu = scan_u(p);
+			while (*p != '\0' && !isspace((unsigned char) *p))
+				p++;
+			while (isspace((unsigned char) *p))
+				p++;
+		}
+	}
+	if (*p == '\0')
+		goto err;
+
+	/* PS functions */
+	p = strdup(p);
+	tblt->head = p;
+	while (*p != '\0' && !isspace((unsigned char) *p))
+		p++;
+	if (*p == '\0')
+		goto err;
+	*p++ = '\0';
+	while (isspace((unsigned char) *p))
+		p++;
+	tblt->note = p;
+	while (*p != '\0' && !isspace((unsigned char) *p))
+		p++;
+	if (*p != '\0') {
+		*p++ = '\0';
+		while (isspace((unsigned char) *p))
+			p++;
+		tblt->bar = p;
+		while (*p != '\0' && !isspace((unsigned char) *p))
+			p++;
+		if (*p != '\0')
+			goto err;
+	}
+
+	/* memorize the definition */
+	if (n >= 0)
+		tblts[n] = tblt;
+	return tblt;
+err:
+	error(1, 0, "Wrong values in %%%%tablature");
+	return 0;
+}
+
 /* -- parse a format line -- */
 void interpret_fmt_line(char *w,		/* keyword */
 			char *p,		/* argument */
@@ -667,6 +811,13 @@ void interpret_fmt_line(char *w,		/* keyword */
 				user_ps_add(p);
 			return;
 		}
+		break;
+	case 't':
+		if (strcmp(w, "tablature") == 0) {
+			tblt_parse(p);
+			return;
+		}
+		break;
 	}
 	for (fd = format_tb; fd->name; fd++)
 		if (strcmp(w, fd->name) == 0)
@@ -714,7 +865,7 @@ void interpret_fmt_line(char *w,		/* keyword */
 			cfmt.textoption = get_textopt(p);
 		else	sscanf(p, "%d", (int *) fd->v);
 		switch (fd->subtype) {
-		case 1:
+		case 1:					/* 'encoding' */
 			if (isdigit(*p)
 			    && (unsigned) cfmt.encoding > MAXENC) {
 				error(1, 0,
@@ -734,7 +885,7 @@ void interpret_fmt_line(char *w,		/* keyword */
 		case 2:
 			nbar = nbar_rep = cfmt.measurefirst;
 			break;
-		case 4:
+		case 4:				/* 'textoption' */
 			if (cfmt.textoption < 0) {
 				error(1, 0,
 				      "Bad 'textoption' value '%s'",
@@ -927,7 +1078,19 @@ int read_fmt_file(char *fn)
 				if (!fgets(p, sizeof line, fp))
 					break;
 #if 1
-				p[strlen(p) - 1] = '\0';
+				{
+					int i;
+					char c;
+
+					i = strlen(p);
+					while (--i > 0) {
+						c = p[i];
+						if (!isspace((unsigned char) c)) {
+							p[i + 1] = '\0';
+							break;
+						}
+					}
+				}
 #else
 				p = clean_line(p);
 #endif
