@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2007 Jean-Franois Moine
+ * Copyright (C) 1998-2008 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -205,8 +205,7 @@ static char *trim_title(char *p, int first);
 /* -- print message for internal error and maybe stop -- */
 void bug(char *msg, int fatal)
 {
-	error(1, 0, "This cannot happen!\n"
-	       "Internal error: %s.\n", msg);
+	error(1, 0, "Internal error: %s.", msg);
 	if (fatal) {
 		fprintf(stderr, "Emergency stop.\n\n");
 		exit(3);
@@ -641,8 +640,10 @@ void add_to_text_block(char *s, int job)
 				PUT1("%.1f 0 M ", strlw);
 				put_str(s, A_RIGHT);
 			}
-		} else	bskip(baseskip * 0.5);
-		buffer_eob();
+		} else {
+			bskip(baseskip * 0.5);
+			buffer_eob();
+		}
 		strtw = 0;
 		return;
 	}
