@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2007 Jean-Franois Moine
+ * Copyright (C) 1998-2008 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -127,7 +127,7 @@ static char ps_head[] =
 	/* str x y bnum - tuplet number / ratio */
 	"/bnum{M/Times-Italic 12 selectfont showc}!\n"
 	/* same with clearing below the number */
-	"/bnumb{	currentgray /Times-Italic 12 selectfont\n"
+	"/bnumb{	currentgray/Times-Italic 12 selectfont\n"
 	"	3 index stringwidth pop 4 add\n"
 	"	dup .5 mul neg 4 index add 3 index 3 -1 roll 8\n"
 	"	1.0 setgray rectfill setgray M showc}!\n"
@@ -312,7 +312,7 @@ static char ps_head[] =
 	"	1.3 0.7 2.9 -1.3 0.7 -2 RC\n"
 	"	fill\n"
 	"	M 0.8 SLW -6 1.2 RM 12.6 12.6 RL stroke\n"
-	"	7 add exch -6 add exch 1.2 0 360 arc fill\n"
+	"	7 add exch 6 sub exch 1.2 0 360 arc fill\n"
 	"	8 add exch 6 add exch 1.2 0 360 arc fill}!\n"
 
 	/* w x y cresc - crescendo */
@@ -402,15 +402,12 @@ static char ps_head[] =
 	"	0 -6 -5 -6 -5 0 RC\n"
 	"	2 add M 0 -4 RL stroke}!\n"
 
-	/* y hl - helper line at height y */
-	"/hl{	.8 SLW x -6 add exch M\n"
-	"	12 0 RL stroke}!\n"
-	/* y hl1 - longer helper line */
-	"/hl1{	.8 SLW x -7 add exch M\n"
-	"	14 0 RL stroke}!\n"
-	/* y hl2 - more longer helper line */
-	"/hl2{	.7 SLW x -9 add exch M\n"
-	"	18 0 RL stroke}!\n"
+	/* x y hl - ledger line */
+	"/hl{	.8 SLW M -6 0 RM 12 0 RL stroke}!\n"
+	/* x y hl1 - longer ledger line */
+	"/hl1{	.8 SLW M -7 0 RM 14 0 RL stroke}!\n"
+	/* x y hl2 - more longer ledger line */
+	"/hl2{	.7 SLW M -9 0 RM 18 0 RL stroke}!\n"
 
 	/* -- accidentals -- */
 	/* x y sh0 - sharp sign */
@@ -480,34 +477,33 @@ static char ps_head[] =
 	"/ft513{2 copy gsave -1 1 scale exch neg 3 add exch M ftx grestore\n"
 	"	M 1.5 0 RM ftx}!\n"
 
-#if 1
 	/* accidentals in strings */
 	"/accfont{\n"
 	"	/CharStrings CharStrings dup length 3 add dict copy def\n"
 	"	FontMatrix 0 get 1 eq{\n"
-	"	 CharStrings /sharpchar{pop\n"
-	"		.65 0 0 -.05 .65 .75 setcachedevice\n"
-	"		.056 dup scale 5.8 7 sh0}bind put\n"
-	"	 CharStrings /flatchar{pop\n"
-	"		.60 0 0 0 .60 .75 setcachedevice\n"
+	"	 CharStrings/sharpchar{pop\n"
+	"		.60 0 0 -.10 .60 .75 setcachedevice\n"
+	"		.056 dup scale 5.8 6 sh0}bind put\n"
+	"	 CharStrings/flatchar{pop\n"
+	"		.60 0 0 0 .60 .78 setcachedevice\n"
 	"		.056 dup scale 5.8 5 ft0}bind put\n"
-	"	 CharStrings /natchar{pop\n"
-	"		.60 0 0 0 .60 .75 setcachedevice\n"
-	"		.056 dup scale 5.8 7 nt0}bind put\n"
+	"	 CharStrings/natchar{pop\n"
+	"		.60 0 0 -.10 .60 .75 setcachedevice\n"
+	"		.056 dup scale 5.8 6 nt0}bind put\n"
 	"	}{\n"
-	"	 CharStrings /sharpchar{pop\n"
-	"		650 0 0 -50 650 750 setcachedevice\n"
-	"		56 dup scale 5.8 7 sh0}bind put\n"
-	"	 CharStrings /flatchar{pop\n"
-	"		600 0 0 0 600 750 setcachedevice\n"
+	"	 CharStrings/sharpchar{pop\n"
+	"		600 0 0 -100 600 750 setcachedevice\n"
+	"		56 dup scale 5.8 6 sh0}bind put\n"
+	"	 CharStrings/flatchar{pop\n"
+	"		600 0 0 0 600 780 setcachedevice\n"
 	"		56 dup scale 5.8 5 ft0}bind put\n"
-	"	 CharStrings /natchar{pop\n"
-	"		600 0 0 0 600 750 setcachedevice\n"
-	"		56 dup scale 5.8 7 nt0}bind put\n"
+	"	 CharStrings/natchar{pop\n"
+	"		600 0 0 -100 600 750 setcachedevice\n"
+	"		56 dup scale 5.8 6 nt0}bind put\n"
 	"	 }ifelse\n"
 	/*if RoPS and Font Type 3, change BuildChar*/
 	"	product(RoPS)eq FontType 3 eq and{\n"
-	"		/TTBuildChar /BuildChar load def\n"
+	"		/TTBuildChar/BuildChar load def\n"
 	"		/BuildChar{1 index begin\n"
 	"			dup Encoding exch get\n"
 	"			CharStrings exch get\n"
@@ -520,21 +516,7 @@ static char ps_head[] =
 	"		}bind def\n"
 	"	}if\n"
 	"	}!\n"
-#else
-	/* accidentals in guitar chord */
-	"/sharp_glyph{\n"
-	"	fh .4 mul 0 RM currentpoint\n"
-	"	gsave T fh .08 mul dup scale 0 7 sh0 grestore\n"
-	"	fh .4 mul 0 RM}!\n"
-	"/flat_glyph{\n"
-	"	fh .4 mul 0 RM currentpoint\n"
-	"	gsave T fh .08 mul dup scale 0 5 ft0 grestore\n"
-	"	fh .4 mul 0 RM}!\n"
-	"/nat_glyph{\n"
-	"	fh .4 mul 0 RM currentpoint\n"
-	"	gsave T fh .08 mul dup scale 0 7 nt0 grestore\n"
-	"	fh .4 mul 0 RM}!\n"
-#endif
+
 	/* str gcshow - guitar chord */
 	"/gcshow{show}!\n"
 	/* x y w h box - draw a box */
@@ -624,7 +606,7 @@ static char ps_head[] =
 	"	-1.7 2 RM 10.5 -1 12 4.5 12 3.5 RC\n"
 	"	0 -1 -3.5 -5.5 -8.5 -5.5 RC fill\n"
 	"	3 SLW M 0 2 RM\n"
-	"	0 exch neg -8 add RL currentpoint stroke\n"
+	"	0 exch neg 8 sub RL currentpoint stroke\n"
 	"	dlw M -1.7 0 RM\n"
 	"	10.5 1 12 -4.5 12 -3.5 RC\n"
 	"	0 1 -3.5 5.5 -8.5 5.5 RC fill}!\n"
@@ -731,32 +713,12 @@ static char ps_head[] =
 	"/gua{x y M -1 4 RM RL stroke}!\n"
 	"/gda{x y M -5 -4 RM RL stroke}!\n"
 
-	/* y ghl - grace note helper line */
-	"/ghl{	.6 SLW x -3 add exch M\n"
-	"	6 0 RL stroke}!\n"
+	/* x y ghl - grace note ledger line */
+	"/ghl{	.6 SLW M -3 0 RM 6 0 RL stroke}!\n"
 
 	/* x1 y2 x2 y2 x3 y3 x0 y0 gsl - grace note slur */
 	"/gsl{dlw M curveto stroke}!\n"
 
-	/* tin whistle */
-	"/tw_head{/Helvetica 8 selectfont\n"
-	"	0 -45 M 90 rotate(WHISTLE)show -90 rotate\n"
-	"	/Helvetica-Bold 36 selectfont\n"
-	"	0 -45 M show .5 SLW newpath}!\n"
-	"/tw_under{\n"
-	"	1 index 2.5 sub -4 M 2.5 -2.5 RL 2.5 2.5 RL\n"
-	"	-2.5 -2.5 RM 0 6 RL stroke}!\n"
-	"/tw_over{\n"
-	"	1 index 2.5 sub -3 M 2.5 2.5 RL 2.5 -2.5 RL\n"
-	"	-2.5 2.5 RM 0 -6 RL stroke}!\n"
-	"/tw_0{7 sub 2 copy 3.5 sub 3 0 360 arc stroke}!\n"
-	"/tw_1{7 sub 2 copy 3.5 sub 2 copy 3 90 270 arc fill 3 270 90 arc stroke}!\n"
-	"/tw_2{7 sub 2 copy 3.5 sub 3 0 360 arc fill}!\n"
-	"/tw_p{pop -55 M 0 6 RL -3 -3 RM 6 0 RL stroke}!\n"
-	"/tw_pp{	pop 3 sub -53.5 M 6 0 RL\n"
-	"	-1.5 -1.5 RM 0 3 RL\n"
-	"	-3 0 RM 0 -3 RL stroke}!\n"
-
 	/* x y showerror */
 	"/showerror{	gsave 1 0.7 0.7 setrgbcolor 2.5 SLW 2 copy newpath\n"
 	"	30 0 360 arc stroke grestore}!\n"
@@ -880,15 +842,15 @@ void define_encoding(int enc,		/* index */
 			"/%s ISOLatin1Encoding dup length array copy def\n",
 			ename);
 		fprintf(fout,
-			"%s dup 8#201 /sharpchar put\n"
-			"dup 8#202 /flatchar put\n"
-			"8#203 /natchar put\n",
+			"%s dup 8#201/sharpchar put\n"
+			"dup 8#202/flatchar put\n"
+			"8#203/natchar put\n",
 			ename);
 	}
 	fprintf(fout, "/mkfontext%d{\n"
 		"	findfont dup length\n"
-		"	 product(RoPS)eq{1 add}if\n"
-		"	 dict begin\n"
+		"	product(RoPS)eq{1 add}if\n"
+		"	dict begin\n"
 		"		{1 index/FID ne{def}{pop pop}ifelse}forall\n",
 		enc);
 	fprintf(fout,
@@ -944,7 +906,7 @@ void define_symbols(void)
 		"		1 sub{	currentpoint\n"
 		"			0.9 -3.7 9.1 -6.4 6 -12.4 RC\n"
 		"			1 5.4 -4.2 8.4 -6 8.4 RC\n"
-		"			fill -5.4 add M\n"
+		"			fill 5.4 sub M\n"
 		"		}repeat\n"
 		"		1.2 -3.2 9.6 -5.7 5.6 -14.6 RC\n"
 		"		1.6 5.4 -1 10.2 -5.6 11.4 RC\n"
@@ -984,12 +946,12 @@ void define_symbols(void)
 		"		}repeat\n"
 		"	}{\n"
 		"		dlw x y M %.1f %.1f RM\n"
-		"		-%.1f add 0 exch RL currentpoint stroke\n"
+		"		%.1f sub 0 exch RL currentpoint stroke\n"
 		"		M{	currentpoint\n"
 		"			7 -%.1f RL\n"
 		"			0 -%.1f RL\n"
 		"			-7 %.1f RL\n"
-		"			fill -5.4 add M\n"
+		"			fill 5.4 sub M\n"
 		"		}repeat\n"
 		"	}ifelse}!\n",
 		STEM_XOFF, STEM_YOFF, STEM_YOFF,
@@ -1018,7 +980,7 @@ void define_symbols(void)
 		"		{	currentpoint\n"
 		"			1 -3.2 5.6 -2.8 3.2 -8 RC\n"
 		"			1.4 4.8 -2.4 5.4 -3.2 5.2 RC\n"
-		"			fill -3.5 add M\n"
+		"			fill 3.5 sub M\n"
 		"		}repeat\n"
 		"	  }ifelse}!\n",
 		GSTEM_XOFF);
@@ -1045,7 +1007,7 @@ void define_symbols(void)
 		"	0 exch RL currentpoint stroke\n"
 		"	M{	currentpoint\n"
 		"		3 -1.5 RL 0 -2 RL -3 1.5 RL\n"
-		"		closepath fill -3 add M\n"
+		"		closepath fill 3 sub M\n"
 		"	}repeat}!\n",
 		GSTEM_XOFF);
 }
